local modName, mod = ...

local myeb

--------------------------------------------------------------------------------

function mod.ShowEB()
	myeb:RegisterEvent("UPDATE_SHAPESHIFT_FORM")
	myeb:RegisterEvent("PLAYER_TALENT_UPDATE")
	myeb:RegisterEvent("UNIT_AURA")
	myeb:RegisterEvent("ECLIPSE_DIRECTION_CHANGE")
	myeb:Show()
end

function mod.HideEB()
	myeb:UnregisterAllEvents()
	myeb:Hide()
end



-- custom druid bar code
--------------------------------------------------------------------------------


-- defines
--------------------------------------------------------------------------------
local WIDTH = 120
local HEIGHT = 21
local X50 = 60
local X100 = 120
local GLOW_SIZE = 40
local PADDING = 2
local ICON_GLOW = [[Interface\Cooldown\starburst]]
local BUFF_SOLAR = GetSpellInfo(171744) -- solar peak
local BUFF_LUNAR = GetSpellInfo(171743) -- lunar peak
local ICON_SUN	= [[Interface\AddOns\clcEclipseBar\media\sun]]
local ICON_MOON	= [[Interface\AddOns\clcEclipseBar\media\moon]]
local TEX_BAR = [[Interface\AddOns\clcEclipseBar\media\glamour_mirror]]
local TEX_MARKER = [[Interface\AddOns\clcEclipseBar\media\arrows]]
-- utility vars
--------------------------------------------------------------------------------
local lastPower = -1000

-- turn off the glows
local function EB_Deactivate()
	myeb.sun:SetAlpha(1)
	myeb.sun2:Hide()
	myeb.sun_glow:Hide()
	myeb.solar_glow:Hide()
	myeb.moon:SetAlpha(1)
	myeb.moon2:Hide()
	myeb.moon_glow:Hide()
	myeb.lunar_glow:Hide()
end

-- check for eclipse buffs and toggle glows
local function EB_CheckBuffs()
	local hasEclipseSolar = UnitBuff("player", BUFF_SOLAR, nil, "PLAYER")
	local hasEclipseLunar = UnitBuff("player", BUFF_LUNAR, nil, "PLAYER")
	
	if hasEclipseSolar then
		myeb.sun:SetAlpha(1)
		myeb.sun2:Show()
		myeb.sun_glow:Show()
		myeb.solar_glow:Show()
	elseif hasEclipseLunar then
		myeb.moon:SetAlpha(1)
		myeb.moon2:Show()
		myeb.moon_glow:Show()
		myeb.lunar_glow:Show()
	else
		EB_Deactivate()
	end
end

-- check if bar should be hidden or shown
local function EB_UpdateShown()
	local form = GetShapeshiftFormID()
	if form == MOONKIN_FORM or not form then
		if GetSpecialization() == 1 then
			myeb:Show()
			return
		end
	end
	lastPower = -1000
	myeb:Hide()
end

-- update bar position if power value changes
local function EB_OnUpdate()
	local power = UnitPower( "player", SPELL_POWER_ECLIPSE )
	if power == lastPower then return end
	
	myeb.powerText:SetText(abs(power))
	
	power = power * (X50 / 100)
	
	myeb.solar_bar:SetPoint("RIGHT", myeb, "CENTER", power, 0)
	myeb.solar_bar:SetTexCoord(0, 1 - ((X50 - power) / X100), 0, 1)
	myeb.solar_bar:SetGradientAlpha("HORIZONTAL", 1, 1, 0, 0, 1, 1, 0, (X50 + power) / X100)
	
	myeb.lunar_bar:SetPoint("LEFT", myeb, "CENTER", power, 0)
	myeb.lunar_bar:SetTexCoord((X50 + power) / X100, 1, 0, 1)
	myeb.lunar_bar:SetGradientAlpha("HORIZONTAL", 0, 0.5, 1, (X50 - power) / X100, 0, 0.5, 1, 0)
	
	myeb.marker:SetPoint("CENTER", myeb, "CENTER", power, 0)
	
	lastPower = power
end

-- event handling
function EB_OnEvent(self, event, arg1)
	if event == "UNIT_AURA" and arg1 == "player" then
		EB_CheckBuffs()
	elseif event == "ECLIPSE_DIRECTION_CHANGE" then
		if arg1 == "sun" then
			myeb.marker:SetTexCoord(0.546875, 0.859375, 0.000000, 0.984375) -- >
		else
			myeb.marker:SetTexCoord(0.109375, 0.421875, 0.000000, 0.984375)	-- <
		end
	else
		EB_UpdateShown()
	end
end

-- activate events and setup initial bars
local function EB_OnShow()
	EB_Deactivate()
	
	local direction = GetEclipseDirection()
	if direction == "sun" then
		myeb.marker:SetTexCoord(0.546875, 0.859375, 0.000000, 0.984375) -- >
	elseif direction == "moon" then
		myeb.marker:SetTexCoord(0.109375, 0.421875, 0.000000, 0.984375)	-- <
	else
		myeb.marker:SetTexCoord(0, 0, 0, 0)
	end
	
	EB_CheckBuffs()
end

-- create the bar :p
function mod.EBCreate()
	local t, fs

	myeb = CreateFrame("Frame", "clcEclipseBarFrame", UIParent)
	myeb:SetSize(WIDTH, HEIGHT)
	myeb:Hide()
	
	-- bg
	t = myeb:CreateTexture(nil, "BACKGROUND")
	myeb.bg = t
	t:SetTexture(TEX_BAR)
	t:SetGradientAlpha("HORIZONTAL", 1, 1, 1, 0.3, 1, 1, 1, 0.3)
	t:SetSize(WIDTH, HEIGHT)
	t:SetPoint("CENTER")
	
	-- solar bar
	t = myeb:CreateTexture(nil, "ARTWORK")
	myeb.solar_bar = t
	t:SetTexture(TEX_BAR)
	t:SetSize(WIDTH, HEIGHT)
	t:SetPoint("LEFT", myeb, "CENTER", -X50, 0)
	-- solar bar blow
	t = myeb:CreateTexture(nil, "OVERLAY")
	myeb.solar_glow = t
	t:SetTexture(ICON_GLOW)
	t:SetBlendMode("ADD")
	t:SetGradientAlpha("HORIZONTAL", 1, 1, 0.5, 0, 1, 1, 0.5, 0.7)
	t:SetTexCoord(0.5, 1, 0, 1)
	t:SetSize(WIDTH, 2 * HEIGHT)
	t:SetPoint("LEFT", myeb, "CENTER")
	
	
	-- lunar bar
	t = myeb:CreateTexture(nil, "ARTWORK")
	myeb.lunar_bar = t
	t:SetTexture(TEX_BAR)
	t:SetSize(WIDTH, HEIGHT)
	t:SetPoint("RIGHT", myeb, "CENTER", X50, 0)
	-- lunar bar glow
	t = myeb:CreateTexture(nil, "OVERLAY")
	myeb.lunar_glow = t
	t:SetTexture(ICON_GLOW)
	t:SetBlendMode("ADD")
	t:SetGradientAlpha("HORIZONTAL", 0.6, 0.6, 1, 1, 0.6, 0.6, 1, 0)
	t:SetTexCoord(0, 0.5, 0, 1)
	t:SetSize(WIDTH, 2 * HEIGHT)
	t:SetPoint("RIGHT", myeb, "CENTER")
	
	-- sun icon
	t = myeb:CreateTexture(nil, "ARTWORK")
	myeb.sun = t	
	t:SetTexture(ICON_SUN)
	t:SetSize(HEIGHT, HEIGHT)
	t:SetPoint("LEFT", myeb, "RIGHT", 1, 0)
	-- double effect when activated
	t = myeb:CreateTexture(nil, "ARTWORK", nil, 1)
	myeb.sun2 = t
	t:SetTexture(ICON_SUN)
	t:SetBlendMode("ADD")
	t:SetSize(HEIGHT, HEIGHT)
	t:SetPoint("LEFT", myeb, "RIGHT", 1, 0)
	t:Hide()
	-- glow
	t = myeb:CreateTexture(nil, "OVERLAY")
	myeb.sun_glow = t
	t:SetTexture(ICON_GLOW)
	t:SetBlendMode("ADD")
	t:SetVertexColor(1, 1, 0.5, 0.7)
	t:ClearAllPoints()
	t:SetSize(GLOW_SIZE, GLOW_SIZE)
	t:SetPoint("CENTER", myeb.sun)
	t:Hide()
	
	-- moon icon
	t = myeb:CreateTexture(nil, "ARTWORK")
	myeb.moon = t
	t:SetTexture(ICON_MOON)
	t:SetSize(HEIGHT, HEIGHT)
	t:SetPoint("RIGHT", myeb, "LEFT", -1, 0)
	-- double effect when activated
	t = myeb:CreateTexture(nil, "ARTWORK", nil, 1)
	myeb.moon2 = t
	t:SetTexture(ICON_MOON)
	t:SetBlendMode("ADD")
	t:SetSize(HEIGHT, HEIGHT)
	t:SetPoint("RIGHT", myeb, "LEFT", -1, 0)
	t:Hide()
	t = myeb:CreateTexture(nil, "OVERLAY")
	myeb.moon_glow = t
	t:SetTexture(ICON_GLOW)
	t:SetBlendMode("ADD")
	t:SetVertexColor(0.6, 0.6, 1, 1)
	t:SetSize(GLOW_SIZE, GLOW_SIZE)
	t:SetPoint("CENTER", myeb.moon)
	t:Hide()
	
	-- marker
	t = myeb:CreateTexture(nil, "OVERLAY")
	myeb.marker = t
	t:SetTexture(TEX_MARKER)
	t:SetSize(15, 25)
	t:SetPoint("CENTER", 0, 00)
	
	fs = myeb:CreateFontString(nil, "OVERLAY", "TextStatusBarText")
	myeb.powerText = fs
	fs:SetText(0)
	fs:SetPoint("CENTER", 0, 0)
	
	myeb:SetScript("OnEvent", EB_OnEvent)
	myeb:SetScript("OnUpdate", EB_OnUpdate)
	myeb:SetScript("OnShow", EB_OnShow)

	myeb:RegisterEvent("UPDATE_SHAPESHIFT_FORM")
	myeb:RegisterEvent("PLAYER_TALENT_UPDATE")
	myeb:RegisterEvent("UNIT_AURA")
	myeb:RegisterEvent("ECLIPSE_DIRECTION_CHANGE")
end

